GoldHoarder = {}
GoldHoarderSaved = {
	FrameScale = 1,
	FrameX = 0,
	FrameY = 0,
	
	ShowFrame = 1,
	
	AutoRepair = 0,
	AutoSellTrash = 0,
	
	Money = {},
	Class = {}
}

local VarsLoaded

local Realm = GetRealmName()
local CharName = UnitName("player")
local _, Class = UnitClass("player")

local MoneyLast
local Incomings = 0
local Outgoings = 0
local VendorIncomings = 0
local VendorOutgoings = 0

local FrameIsMoving

local ClassHexColour = {
	["DEATHKNIGHT"] = "C41F3B",
	["DEMONHUNTER"] = "A330C9",
	["DRUID"] = "FF7D0A",
	["HUNTER"] = "ABD473",
	["MAGE"] = "40C7EB",
	["MONK"] = "00FF96",
	["PALADIN"] = "F58CBA",
	["PRIEST"] = "FFFFFF",
	["ROGUE"] = "FFF569",
	["SHAMAN"] = "0070DE",
	["WARLOCK"] = "8787ED",
	["WARRIOR"] = "C79C6E",
}

function GoldHoarder.PrintProfitInfo()
	print("|cff0099ffWyeSoft Gold Hoarder:|r Since login:")
	print("|cff00ff00Incomings:|r " .. GetCoinTextureString(Incomings))
	print("|cffff0000Outgoings:|r " .. GetCoinTextureString(Outgoings))
	local Profit = Incomings - Outgoings
	if Profit >= 0 then
		print("|cff00ff00Profit:|r " .. GetCoinTextureString(Profit))
	else
		print("|cffff0000Loss:|r " .. GetCoinTextureString(-1 * Profit))
	end
	
	print(CharName .. "'s money: " .. GetCoinTextureString(GetMoney()))
	
	local Faction = UnitFactionGroup("player")
	if Faction == "Alliance" or Faction == "Horde" then
		local TotalMoney = 0
		for _, CharMoney in pairs(GoldHoarderSaved.Money[Realm][Faction]) do
			TotalMoney = TotalMoney + CharMoney
		end
		
		print(Realm .. " " .. Faction .. " total: " .. GetCoinTextureString(TotalMoney))
	end
end

function GoldHoarder.PrintCharInfo()
	local Faction = UnitFactionGroup("player")
	if Faction == "Alliance" or Faction == "Horde" then
		print("|cff0099ffWyeSoft Gold Hoarder:|r " .. Faction .. " characters on " .. Realm .. ":")
		
		local TotalMoney = 0
		for CountChar, CharMoney in pairs(GoldHoarderSaved.Money[Realm][Faction]) do
			TotalMoney = TotalMoney + CharMoney
			print("|cff" .. ClassHexColour[GoldHoarderSaved.Class[Realm][CountChar]] .. CountChar .. "|r: " .. GetCoinTextureString(CharMoney))
		end
		
		print("Total: " .. GetCoinTextureString(TotalMoney))
	else
		print("|cff0099ffWyeSoft Gold Hoarder:|r This feature won't be available until you have selected a faction.")
	end
end

function GoldHoarder.FrameOnLoad(Frame)
	Frame:RegisterEvent("ADDON_LOADED")
	Frame:RegisterEvent("PLAYER_MONEY")
	Frame:RegisterEvent("MERCHANT_SHOW")
	Frame:RegisterEvent("MERCHANT_CLOSED")
	Frame:RegisterEvent("TRAINER_SHOW")
	Frame:RegisterEvent("TRAINER_CLOSED")
	Frame:RegisterEvent("MAIL_SHOW")
	Frame:RegisterEvent("MAIL_CLOSED")
	Frame:RegisterEvent("AUCTION_HOUSE_SHOW")
	Frame:RegisterEvent("AUCTION_HOUSE_CLOSED")
end

function GoldHoarder.FrameOnEvent(Frame, Event, ...)
	local Faction = UnitFactionGroup("player")
	local MoneyNow = GetMoney()
	
	if Event == "ADDON_LOADED" then
		VarsLoaded = true
		
		if not GoldHoarderSaved.Class[Realm] then GoldHoarderSaved.Class[Realm] = {} end
		GoldHoarderSaved.Class[Realm][CharName] = Class
		
		MoneyLast = MoneyNow
		
		if GoldHoarderSaved.ShowFrame == 1 then
			Frame:SetScale(GoldHoarderSaved.FrameScale)
			Frame:ClearAllPoints()
			Frame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", GoldHoarderSaved.FrameX, GoldHoarderSaved.FrameY)
			
			Frame:Show()
		end
	end
	if not VarsLoaded then return end

	if Event == "ADDON_LOADED" or Event == "PLAYER_MONEY" then
		if Faction == "Alliance" or Faction == "Horde" then
			if not GoldHoarderSaved.Money[Realm] then GoldHoarderSaved.Money[Realm] = {} end
			if not GoldHoarderSaved.Money[Realm][Faction] then GoldHoarderSaved.Money[Realm][Faction] = {} end
			GoldHoarderSaved.Money[Realm][Faction][CharName] = MoneyNow
		end
		
		local Change = MoneyNow - MoneyLast
		if Change >= 0 then
			Incomings = Incomings + Change
			VendorIncomings = VendorIncomings + Change
		else
			Outgoings = Outgoings - Change
			VendorOutgoings = VendorOutgoings - Change
		end
		
		MoneyLast = MoneyNow
		
		GoldHoarderFrameCharMoney:SetText(GetCoinTextureString(MoneyNow))
		
		local TotalMoney = 0
		if Faction == "Alliance" or Faction == "Horde" then
			for _, CharMoney in pairs(GoldHoarderSaved.Money[Realm][Faction]) do
				TotalMoney = TotalMoney + CharMoney
			end
		else
			TotalMoney = MoneyNow
		end
		GoldHoarderFrameRealmMoney:SetText(GetCoinTextureString(TotalMoney))
	elseif Event == "MERCHANT_SHOW" or Event == "TRAINER_SHOW" or Event == "MAIL_SHOW" or Event == "AUCTION_HOUSE_SHOW" then
		VendorIncomings = 0
		VendorOutgoings = 0
		
		if Event == "MERCHANT_SHOW" then
			if GoldHoarderSaved.AutoSellTrash == 1 then
				for Bag = 0, 4 do
					for Slot = 1, GetContainerNumSlots(Bag) do
						local ItemLink = GetContainerItemLink(Bag, Slot)
						if ItemLink and string.sub(ItemLink, 1, 10) == "|cff9d9d9d" then UseContainerItem(Bag, Slot) end
					end
				end
			end
			
			if GoldHoarderSaved.AutoRepair == 1 and CanMerchantRepair() then RepairAllItems() end
		end
	elseif ( Event == "MERCHANT_CLOSED" or Event == "TRAINER_CLOSED" or Event == "MAIL_CLOSED" or Event == "AUCTION_HOUSE_CLOSED" ) and (VendorIncomings > 0 or VendorOutgoings > 0)then
		if GoldHoarderSaved.NoVendorReports ~= 1 then
			local Title
			if Event == "MAIL_CLOSED" then
				Title = "Mailbox"
			elseif Event == "AUCTION_HOUSE_CLOSED" then
				Title = "Auction House"
			else
				Title = "Vendor"
			end
			
			print("|cff0099ffWyeSoft Gold Hoarder:|r " .. Title .. ":")
			print("|cff00ff00Incomings:|r " .. GetCoinTextureString(VendorIncomings))
			print("|cffff0000Outgoings:|r " .. GetCoinTextureString(VendorOutgoings))
			local Profit = VendorIncomings - VendorOutgoings
			if Profit >= 0 then
				print("|cff00ff00Profit:|r " .. GetCoinTextureString(Profit))
			else
				print("|cffff0000Loss:|r " .. GetCoinTextureString(-1 * Profit))
			end
		end
		
		VendorIncomings = 0
		VendorOutgoings = 0
	end
end

function GoldHoarder.FrameOnMouseDown(Frame, Button)
	if (Button == "LeftButton" or Button == "RightButton") and IsShiftKeyDown() and not FrameIsMoving then
		Frame:StartMoving()
		FrameIsMoving = true
	end
end

function GoldHoarder.FrameOnMouseUp(Frame, Button)
	if (Button == "LeftButton" or Button == "RightButton") then
		if FrameIsMoving then
			Frame:StopMovingOrSizing()
			FrameIsMoving = false
			
			GoldHoarderSaved.FrameX = Frame:GetLeft()
			GoldHoarderSaved.FrameY = Frame:GetBottom()
		elseif GoldHoarderFrameCharMoney:IsMouseOver() then
			GoldHoarder.PrintProfitInfo()
		elseif GoldHoarderFrameRealmMoney:IsMouseOver() then
			GoldHoarder.PrintCharInfo()
		end
	end
end

function GoldHoarder.FrameOnMouseWheel(Frame, Direction)
	if IsShiftKeyDown() then
		local NewScale = GoldHoarderSaved.FrameScale + (Direction / 10)
		if NewScale > 0 then
			Frame:SetScale(NewScale)
			
			GoldHoarderSaved.FrameScale = NewScale
			GoldHoarderSaved.FrameX = Frame:GetLeft()
			GoldHoarderSaved.FrameY = Frame:GetBottom()
		end
	end
end

function GoldHoarder.FrameOnHide(Frame)
	if FrameIsMoving then
		Frame:StopMovingOrSizing()
		FrameIsMoving = false
		
		GoldHoarderSaved.FrameX = Frame:GetLeft()
		GoldHoarderSaved.FrameY = Frame:GetBottom()
	end
end

function GoldHoarder.SlashCmds(Msg, EditBox)
	local lMsg
	if Msg then lMsg = string.lower(Msg) end
	
	if string.sub(lMsg, 1, 1) == "h" then
		print("|cff0099ffWyeSoft Gold Hoarder:|r Available commands:")
		print("|cffffff00/GOLD|r  displays incomings, outgoings and profit/loss since login and your current money on both the character you are playing and the realm.")
		print("|cffffff00/GOLD C|r  lists each character's money on the realm and faction you are currently playing on, as well as the total.")
		print("|cffffff00/GOLD F|r  toggles display of Gold Hoarder frame.")
		print("|cffffff00/GOLD R|r  toggles automatic repairing of items when interacting with a vendor that can repair.")
		print("|cffffff00/GOLD T|r  toggles automatic selling of trash (grey) quality items when interacting with a vendor.")
		print("|cffffff00/GOLD V|r  toggles profit/loss reports after interacting with a vendor, a trainer, a mailbox or an auctioneer.")
		print("|cffffff00/GOLD D <Amount>|ALL|r  destroys all trash (grey) quality items worth less than the specified amount, or all trash when entering ALL.  Example: /GOLD D 1G (be careful when using this command, as there is no confirmation box; items will be destroyed immediately).")
		print("|cffffff00/GOLD S <Scale>|r  resizes the Gold Hoarder frame to the given percentage of its default size.  Example: /GOLD S 100  (do not enter a % symbol).")
		print("|cffffff00/GOLD RESET|r  resets size and position of Gold Hoarder frame.")
		print("|cffffff00/GOLD H|r  lists Gold Hoarder's slash commands.")
	elseif string.sub(lMsg, 1, 1) == "f" then
		if GoldHoarderSaved.ShowFrame == 1 then
			GoldHoarderFrame:Hide()
			
			GoldHoarderSaved.ShowFrame = 0
		else
			GoldHoarderFrame:SetScale(GoldHoarderSaved.FrameScale)
			GoldHoarderFrame:ClearAllPoints()
			GoldHoarderFrame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", GoldHoarderSaved.FrameX, GoldHoarderSaved.FrameY)
			
			GoldHoarderFrame:Show()
			
			GoldHoarderSaved.ShowFrame = 1
		end
	elseif string.sub(lMsg, 1, 5) == "reset" then
		GoldHoarderFrame:SetScale(1)
		GoldHoarderFrame:ClearAllPoints()
		GoldHoarderFrame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", 0, 0)
		
		GoldHoarderSaved.FrameScale = 1
		GoldHoarderSaved.FrameX = 0
		GoldHoarderSaved.FrameY = 0
	elseif string.sub(lMsg,1,1) == "r" then
		if GoldHoarderSaved.AutoRepair == 1 then
			GoldHoarderSaved.AutoRepair = 0
			print("|cff0099ffWyeSoft Gold Hoarder:|r Auto-repair disabled")
		else
			GoldHoarderSaved.AutoRepair = 1
			print("|cff0099ffWyeSoft Gold Hoarder:|r Auto-repair enabled")
		end
	elseif string.sub(lMsg,1,1) == "t" then
		if GoldHoarderSaved.AutoSellTrash == 1 then
			GoldHoarderSaved.AutoSellTrash = 0
			print("|cff0099ffWyeSoft Gold Hoarder:|r Auto-selling of trash (grey) items disabled")
		else
			GoldHoarderSaved.AutoSellTrash = 1
			print("|cff0099ffWyeSoft Gold Hoarder:|r Auto-selling of trash (grey) items enabled")
		end
	elseif string.sub(lMsg,1,1) == "v" then
		if GoldHoarderSaved.NoVendorReports == 1 then
			GoldHoarderSaved.NoVendorReports = 0
			print("|cff0099ffWyeSoft Gold Hoarder:|r Vendor reports enabled")
		else
			GoldHoarderSaved.NoVendorReports = 1
			print("|cff0099ffWyeSoft Gold Hoarder:|r Vendor reports disabled")
		end
	elseif string.sub(lMsg, 1, 1) == "c" then
		GoldHoarder.PrintCharInfo()
	elseif string.sub(lMsg, 1, 1) == "s" then
		local NewScale = tonumber(string.sub(Msg, 3))
		if NewScale > 0 then
			NewScale = NewScale / 100
			
			GoldHoarderFrame:SetScale(NewScale)
		
			GoldHoarderSaved.FrameScale = NewScale
			GoldHoarderSaved.FrameX = GoldHoarderFrame:GetLeft()
			GoldHoarderSaved.FrameY = GoldHoarderFrame:GetBottom()
		end
		
		print("|cff0099ffWyeSoft Gold Hoarder:|r Frame scale: " .. (GoldHoarderSaved.FrameScale * 100) .. "%")
	elseif string.sub(lMsg, 1, 1) == "d" then
		local Limit
		if tonumber(string.sub(Msg, 2, -2)) and (string.sub(lMsg, -1) == "c") then
			Limit = tonumber(string.sub(Msg, 2, -2))
		elseif tonumber(string.sub(Msg, 2, -2)) and (string.sub(lMsg, -1) == "s") then
			Limit = tonumber(string.sub(Msg, 2, -2)) * 100
		elseif tonumber(string.sub(Msg, 2, -2)) and (string.sub(lMsg, -1) == "g") then
			Limit = tonumber(string.sub(Msg, 2, -2)) * 10000
		elseif tonumber(string.sub(Msg, 2)) then
			Limit = tonumber(string.sub(Msg, 2))
		elseif string.sub(lMsg, 2) ~= " all" then
			return
		end
		
		for Bag = 0, 4 do
			for Slot = 1, GetContainerNumSlots(Bag) do
				local ItemLink = GetContainerItemLink(Bag, Slot)
				if ItemLink and string.sub(ItemLink, 1, 10) == "|cff9d9d9d" then
					if Limit then
						local ItemID = GetContainerItemID(Bag, Slot)
						local _, Count, _, _, _, _, _ = GetContainerItemInfo(Bag, Slot)
						if not Count then Count = 1 end
						local _, _, _, _, _, _, _, _, _, _, Price = GetItemInfo(ItemID)
						if Count * Price < Limit then
							PickupContainerItem(Bag, Slot)
							DeleteCursorItem()
						end
					else
						PickupContainerItem(Bag, Slot)
						DeleteCursorItem()
					end
				end
			end
		end
	else
		GoldHoarder.PrintProfitInfo()
	end
end

SLASH_GOLDHOARDER1 = "/gold"
SLASH_GOLDHOARDER2 = "/hoarder"
SLASH_GOLDHOARDER3 = "/goldhoarder"
SLASH_GOLDHOARDER4 = "/wyesoftgoldhoarder"
SlashCmdList["GOLDHOARDER"] = GoldHoarder.SlashCmds
